@@ -12,7 +12,7 @@ Python libraries, though it does make heavy use of external packaging commands.
 Visit <https://github.com/datalad/datalad-installer> for more information.
 """
 
-__version__ = "0.3.0"
+__version__ = "0.3.1"
 __author__ = "The DataLad Team and Contributors"
 __author_email__ = "team@datalad.org"
 __license__ = "MIT"
@@ -531,6 +531,8 @@ class DataladInstaller:
         self.conda_stack: List[CondaInstance] = []
         #: A list of commands installed via the instance
         self.new_commands: CommandList = []
+        #: Whether "brew update" has been run
+        self.brew_updated: bool = False
 
     @classmethod
     def register_component(
@@ -668,9 +670,24 @@ class DataladInstaller:
         ok = True
         for name, path in self.new_commands:
             log.info("%s is now installed at %s", name, path)
-            if not os.access(path, os.X_OK):
-                log.error("Cannot execute program!")
+            if not os.path.exists(path):
+                log.error("%s does not exist!", path)
                 ok = False
+            elif not ON_WINDOWS and not os.access(path, os.X_OK):
+                log.error("%s is not executable!", path)
+                ok = False
+            else:
+                try:
+                    sr = subprocess.run(
+                        [str(path), "--help"], stdout=subprocess.DEVNULL
+                    )
+                except Exception as e:
+                    log.error("Failed to run `%s --help`: %s", path, e)
+                    ok = False
+                else:
+                    if sr.returncode != 0:
+                        log.error("`%s --help` command failed!", path)
+                        ok = False
         return 0 if ok else 1
 
     def addenv(self, line: str) -> None:
@@ -1284,11 +1301,22 @@ class HomebrewInstaller(Installer):
         log.info("Extra args: %s", extra_args)
         if kwargs:
             log.warning("Ignoring extra installer arguments: %r", kwargs)
+        if not self.manager.brew_updated:
+            runcmd("brew", "update")
+            self.manager.brew_updated = True
         cmd = ["brew", "install"]
         if extra_args:
             cmd.extend(extra_args)
         cmd.append(package)
-        runcmd(*cmd)
+        try:
+            runcmd(*cmd)
+        except subprocess.CalledProcessError:
+            log.error(
+                "brew command failed; printing diagnostic output for reporting issue"
+            )
+            runcmd("brew", "config")
+            runcmd("brew", "doctor")
+            raise
         ### TODO: Handle variations in this path (Is it "$(brew --prefix)/bin"?)
         log.debug("Installed program directory: /usr/local/bin")
         return Path("/usr/local/bin")
@@ -1373,14 +1401,22 @@ class PipInstaller(Installer):
         )
         runcmd(*cmd)
         user = extra_args is not None and "--user" in extra_args
-        binpath = Path(
-            readcmd(
-                self.python,
-                "-c",
-                "from pip._internal.locations import distutils_scheme;"
-                f" print(distutils_scheme({package!r}, user={user!r})['scripts'], end='')",
+        with tempfile.NamedTemporaryFile("w+", delete=False) as script:
+            # Passing this code to Python with `input` doesn't work for some
+            # reason, so we need to save it as a script instead.
+            print(
+                "try:\n"
+                "    from pip._internal.locations import get_scheme\n"
+                f"    path = get_scheme({package!r}, user={user!r}).scripts\n"
+                "except ImportError:\n"
+                "    from pip._internal.locations import distutils_scheme\n"
+                f"    path = distutils_scheme({package!r}, user={user!r})['scripts']\n"
+                "print(path, end='')\n",
+                file=script,
+                flush=True,
             )
-        )
+            script.close()
+            binpath = Path(readcmd(self.python, script.name))
         log.debug("Installed program directory: %s", binpath)
         return binpath
 
